@@ -3,9 +3,13 @@
 import os
 import re
 import subprocess
+import sys
 from contextlib import contextmanager
+from distutils.util import strtobool
+from os.path import expandvars
 from pathlib import Path
 import logging
+from shutil import which
 from typing import List
 
 logger = logging.getLogger(__file__)
@@ -23,12 +27,34 @@ def _interactive_sub_shell_command(command):
     Raises:
         ValueError: If the subshell command could not be determined.
     """
-    venv_wrapper_shell_filepath = os.environ.get("VENV_MANAGEMENT_SHELL_FILEPATH", "/bin/bash")
-    venvwrapper_setup_filepath = os.environ.get("VENV_MANAGEMENT_SETUP_FILEPATH", "$HOME/.bashrc")
-    # TODO: The $SHELL environment variable is not guaranteed to be set, or to be accurate if it is
-    #  set. Look for a more reliable means of spawning an interactive subshell.
-    #  See: https://stackoverflow.com/questions/3327013/how-to-determine-the-current-shell-im-working-on
-    return f"{venv_wrapper_shell_filepath} -c -i '. {venvwrapper_setup_filepath} && {command}'"
+    preferred_shell_name = os.environ.get("SHELL", "bash")
+    logger.debug("preferred_shell_name from $SHELL = %r", preferred_shell_name)
+    shell_name = expandvars(os.environ.get("VENV_MANAGEMENT_SHELL", preferred_shell_name))
+    logger.debug("shell_name = %r", shell_name)
+    shell_filepath = which(shell_name)
+    logger.debug("shell_filepath = %r", shell_filepath)
+    if shell_filepath is None:
+        raise RuntimeError(f"Could not determine the path to {shell_name}")
+    shell_filepath = Path(shell_filepath)
+    shell_filename = shell_filepath.name
+    logger.debug("shell_filename = %r", shell_filename)
+    rc_filename = f".{shell_filename}rc"
+    logger.debug("rc_filename = %r", rc_filename)
+    rc_filepath = Path.home() / rc_filename
+    logger.debug("rc_filepath = %r", rc_filepath)
+    interactive = strtobool(expandvars(os.environ.get("VENV_MANAGEMENT_INTERACTIVE_SHELL", "no")))
+    logger.debug("interactive = %s", interactive)
+    setup_filepath = Path(expandvars(os.environ.get("VENV_MANAGEMENT_SETUP_FILEPATH", str(rc_filepath))))
+    logger.debug("setup_filepath = %s", setup_filepath)
+    if not setup_filepath.is_file():
+        raise RuntimeError(f"Could not find setup file {setup_filepath}")
+    args = [
+        str(shell_filepath),
+        "-c",  # Run command
+        *(["-i"] if interactive else []),
+        f". {setup_filepath!s} && {command}",
+    ]
+    return args
 
 
 def has_virtualenvwrapper():
@@ -62,8 +88,10 @@ def list_virtual_envs() -> List[str]:
     failed_commands = []
     for lsvirtualenv_command in list(lsvirtualenv_commands):
         command = _interactive_sub_shell_command(lsvirtualenv_command)
-        logger.info(command)
+        logger.debug(command)
         status, output = _getstatusoutput(command)
+        logger.debug("status = %d", status)
+        logger.debug("output = %s", output)
         if status == 0:
             break
         failed_commands.append(lsvirtualenv_command)
@@ -88,14 +116,23 @@ def _getstatusoutput(cmd):
     The exit status for the command can be interpreted
     according to the rules for the function 'wait'.
     """
-    try:
-        data = subprocess.check_output(cmd, shell=True, universal_newlines=True)
-        status = 0
-    except subprocess.CalledProcessError as ex:
-        data = ex.output
-        status = ex.returncode
-    if data[-1:] == '\n':
-        data = data[:-1]
+    logger.debug("command = %r", cmd)
+    process = subprocess.run(
+        cmd,
+        stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE,
+        encoding=sys.getdefaultencoding()
+    )
+    status = process.returncode
+    if status == 0:
+        data = process.stdout
+        if data[-1:] == '\n':
+            data = data[:-1]
+    else:
+        data = (f"STATUS: {status} ; \n"
+                f"STDOUT: {process.stdout} ; \n"
+                f"STDERR: {process.stderr}"
+        )
     return status, data
 
 
